'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.attachNeighborIterationMethods = attachNeighborIterationMethods;

var _errors = require('../errors');

var _utils = require('../utils');

/**
 * Definitions.
 */
/**
 * Graphology Neighbor Iteration
 * ==============================
 *
 * Attaching some methods to the Graph class to be able to iterate over
 * neighbors.
 */
var NEIGHBORS_ITERATION = [{
  name: 'neighbors',
  counter: 'countNeighbors',
  type: 'mixed'
}, {
  name: 'inNeighbors',
  counter: 'countInNeighbors',
  type: 'directed',
  direction: 'in'
}, {
  name: 'outNeighbors',
  counter: 'countOutNeighbors',
  type: 'directed',
  direction: 'out'
}, {
  name: 'inboundNeighbors',
  counter: 'countInboundNeighbors',
  type: 'mixed',
  direction: 'in'
}, {
  name: 'outboundNeighbors',
  counter: 'countOutboundNeighbors',
  type: 'mixed',
  direction: 'out'
}, {
  name: 'directedNeighbors',
  counter: 'countDirectedNeighbors',
  type: 'directed'
}, {
  name: 'undirectedNeighbors',
  counter: 'countUndirectedNeighbors',
  type: 'undirected'
}];

/**
 * Function merging neighbors into the given set iterating over the given object.
 *
 * @param {BasicSet} neighbors - Neighbors set.
 * @param {object}   object    - Target object.
 */
function mergeNeighbors(neighbors, object) {
  if (!object) return;

  for (var neighbor in object) {
    neighbors.add(neighbor);
  }
}

/**
 * Function creating a set of relevant neighbors for the given node.
 *
 * @param  {Graph}        graph     - Target graph.
 * @param  {string}       type      - Type of neighbors.
 * @param  {string}       direction - Direction.
 * @param  {any}          node      - Target node.
 * @return {Set|BasicSet}           - The neighbors set.
 */
function createNeighborSetForNode(graph, type, direction, node) {

  // For this, we need to compute the "structure" index
  graph.computeIndex('structure');

  var neighbors = new Set();

  var nodeData = graph._nodes.get(node);

  if (type === 'mixed' || type === 'directed') {

    if (!direction || direction === 'in') {
      mergeNeighbors(neighbors, nodeData.in);
    }
    if (!direction || direction === 'out') {
      mergeNeighbors(neighbors, nodeData.out);
    }
  }

  if (type === 'mixed' || type === 'undirected') {

    if (!direction || direction === 'in') {
      mergeNeighbors(neighbors, nodeData.undirectedIn);
    }
    if (!direction || direction === 'out') {
      mergeNeighbors(neighbors, nodeData.undirectedOut);
    }
  }

  return neighbors;
}

/**
 * Function creating a set of relevant neighbors for the given bunch of nodes.
 *
 * @param  {string}       name      - Name of the calling method.
 * @param  {Graph}        graph     - Target graph.
 * @param  {string}       type      - Type of neighbors.
 * @param  {string}       direction - Direction.
 * @param  {bunch}        bunch     - Target bunch.
 * @return {Set|BasicSet}           - The neighbors set.
 */
function createNeighborSetForBunch(name, graph, type, direction, bunch) {

  // For this, we need to compute the "structure" index
  graph.computeIndex('structure');

  var neighbors = new Set();

  (0, _utils.overBunch)(bunch, function (node) {
    if (!graph.hasNode(node)) throw new _errors.NotFoundGraphError('Graph.' + name + ': could not find the "' + node + '" node in the graph in the given bunch.');

    var nodeData = graph._nodes.get(node);

    if (type === 'mixed' || type === 'directed') {

      if (!direction || direction === 'in') {
        mergeNeighbors(neighbors, nodeData.in);
      }
      if (!direction || direction === 'out') {
        mergeNeighbors(neighbors, nodeData.out);
      }
    }

    if (type === 'mixed' || type === 'undirected') {

      if (!direction || direction === 'in') {
        mergeNeighbors(neighbors, nodeData.undirectedIn);
      }
      if (!direction || direction === 'out') {
        mergeNeighbors(neighbors, nodeData.undirectedOut);
      }
    }
  });

  return neighbors;
}

/**
 * Function attaching a neighbors array creator method to the Graph prototype.
 *
 * @param {function} Class       - Target class.
 * @param {boolean}  counter     - Should we count or collect?
 * @param {object}   description - Method description.
 */
function attachNeighborArrayCreator(Class, counter, description) {
  var type = description.type,
      direction = description.direction;


  var name = counter ? description.counter : description.name;

  /**
   * Function returning an array or the count of certain neighbors.
   *
   * Arity 1a: Return all of a node's relevant neighbors.
   * @param  {any}   node   - Target node.
   *
   * Arity 1b: Return the union of the relevant neighbors of the given bunch of nodes.
   * @param  {bunch} bunch  - Bunch of nodes.
   *
   * Arity 2: Return whether the two nodes are indeed neighbors.
   * @param  {any}   source - Source node.
   * @param  {any}   target - Target node.
   *
   * @return {array|number} - The neighbors or the number of neighbors.
   *
   * @throws {Error} - Will throw if there are too many arguments.
   */
  Class.prototype[name] = function () {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (args.length === 2) {
      var node1 = args[0],
          node2 = args[1];


      if (counter) throw new _errors.InvalidArgumentsGraphError('Graph.' + name + ': invalid arguments.');

      if (!this.hasNode(node1)) throw new _errors.NotFoundGraphError('Graph.' + name + ': could not find the "' + node1 + '" node in the graph.');

      if (!this.hasNode(node2)) throw new _errors.NotFoundGraphError('Graph.' + name + ': could not find the "' + node2 + '" node in the graph.');

      // Here, we want to assess whether the two given nodes are neighbors
      // NOTE: we could improve performance here
      var neighbors = createNeighborSetForNode(this, type, direction, node1);

      return neighbors.has(node2);
    } else if (args.length === 1) {
      var nodeOrBunch = args[0];

      if (this.hasNode(nodeOrBunch)) {

        // Here, we want to iterate over a node's relevant neighbors
        var _neighbors = createNeighborSetForNode(this, type, direction, nodeOrBunch);

        if (counter) return _neighbors.size;

        return Array.from(_neighbors);
      } else if ((0, _utils.isBunch)(nodeOrBunch)) {

        // Here, we want to iterate over the union of a bunch of nodes'
        // relevant neighbors

        // Note: since we need to keep track of the traversed values
        // to perform union, we can't optimize further and we have to
        // create this intermediary array and return its length when counting.
        var _neighbors2 = createNeighborSetForBunch(name, this, type, direction, nodeOrBunch);

        if (counter) return _neighbors2.size;

        return Array.from(_neighbors2);
      } else {
        throw new _errors.NotFoundGraphError('Graph.' + name + ': could not find the "' + nodeOrBunch + '" node in the graph.');
      }
    }

    throw new _errors.InvalidArgumentsGraphError('Graph.' + name + ': invalid number of arguments (expecting 1 or 2 and got ' + args.length + ').');
  };
}

/**
 * Function attaching every neighbor iteration method to the Graph class.
 *
 * @param {function} Graph - Graph class.
 */
function attachNeighborIterationMethods(Graph) {
  NEIGHBORS_ITERATION.forEach(function (description) {
    attachNeighborArrayCreator(Graph, false, description);
    attachNeighborArrayCreator(Graph, true, description);
  });
}