'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.assign = assign;
exports.createInternalMap = createInternalMap;
exports.firstItemOfSet = firstItemOfSet;
exports.isBunch = isBunch;
exports.isGraph = isGraph;
exports.isPlainObject = isPlainObject;
exports.overBunch = overBunch;
exports.prettyPrint = prettyPrint;
exports.privateProperty = privateProperty;
exports.readOnlyProperty = readOnlyProperty;
exports.uuid = uuid;
/**
 * Graphology Utilities
 * =====================
 *
 * Collection of helpful functions used by the implementation.
 */

/**
 * Very simple Object.assign-like function.
 *
 * @param  {object} target       - First object.
 * @param  {object} [...objects] - Objects to merge.
 * @return {object}
 */
function assign(target) {
  target = target || {};

  for (var _len = arguments.length, objects = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    objects[_key - 1] = arguments[_key];
  }

  for (var i = 0, l = objects.length; i < l; i++) {
    if (!objects[i]) continue;

    for (var k in objects[i]) {
      target[k] = objects[i][k];
    }
  }

  return target;
}

/**
 * Custom Map used internally to coerce keys on vital operations.
 *
 * @return {Map}
 */
function createInternalMap() {
  var map = new Map();

  map.set = function (key, value) {
    key = '' + key;
    return Map.prototype.set.call(this, key, value);
  };

  map.get = function (key) {
    key = '' + key;
    return Map.prototype.get.call(this, key);
  };

  map.has = function (key) {
    key = '' + key;
    return Map.prototype.has.call(this, key);
  };

  return map;
}

/**
 * Function returning the first item of a Set.
 *
 * @param  {object} set - Target set.
 * @return {any}        - The "first" item.
 */
function firstItemOfSet(set) {
  return set.values().next().value;
}

/**
 * Checks whether the given value is a potential bunch.
 *
 * @param  {mixed}   value - Target value.
 * @return {boolean}
 */
function isBunch(value) {
  return !!value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && (Array.isArray(value) || typeof Map === 'function' && value instanceof Map || typeof Set === 'function' && value instanceof Set || !(value instanceof Date) && !(value instanceof RegExp));
}

/**
 * Checks whether the given value is a Graph implementation instance.
 *
 * @param  {mixed}   value - Target value.
 * @return {boolean}
 */
function isGraph(value) {
  return !!value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && typeof value.addUndirectedEdgeWithKey === 'function' && typeof value.dropNode === 'function';
}

/**
 * Checks whether the given value is a plain object.
 *
 * @param  {mixed}   value - Target value.
 * @return {boolean}
 */
function isPlainObject(value) {
  return !!value && (typeof value === 'undefined' ? 'undefined' : _typeof(value)) === 'object' && !Array.isArray(value) && !(value instanceof Date) && !(value instanceof RegExp) && !(typeof Map === 'function' && value instanceof Map) && !(typeof Set === 'function' && value instanceof Set);
}

/**
 * Iterates over the provided bunch.
 *
 * @param {object}   bunch    - Target bunch.
 * @param {function} callback - Function to call.
 */
function overBunch(bunch, callback) {

  // Array iteration
  if (Array.isArray(bunch)) {
    for (var i = 0, l = bunch.length; i < l; i++) {
      var shouldBreak = callback(bunch[i], null) === false;

      if (shouldBreak) break;
    }
  }

  // Map & Set iteration
  else if (typeof bunch.forEach === 'function') {
      var iterator = bunch.entries();

      var _shouldBreak = false,
          step = void 0;

      while (step = iterator.next()) {
        var _step = step,
            value = _step.value,
            done = _step.done;


        if (done) break;

        var k = value[0],
            v = value[1];


        if (v === k) _shouldBreak = callback(v, null) === false;else _shouldBreak = callback(k, v) === false;

        if (_shouldBreak) break;
      }
    }

    // Plain object iteration
    else {
        for (var key in bunch) {
          var attributes = bunch[key];

          var _shouldBreak2 = callback(key, attributes);

          if (_shouldBreak2) break;
        }
      }
}

/**
 * Pretty prints the given integer.
 *
 * @param  {number}  integer - Target integer.
 * @return {string}          - The pretty string.
 */
function prettyPrint(integer) {
  var string = '' + integer;

  var prettyString = '';

  for (var i = 0, l = string.length; i < l; i++) {
    var j = l - i - 1;

    prettyString = string[j] + prettyString;

    if (!((i - 2) % 3)) prettyString = ',' + prettyString;
  }

  return prettyString;
}

/**
 * Creates a "private" property for the given member name by concealing it
 * using the `enumerable` option.
 *
 * @param {object} target - Target object.
 * @param {string} name   - Member name.
 */
function privateProperty(target, name, value) {
  Object.defineProperty(target, name, {
    enumerable: false,
    configurable: false,
    writable: true,
    value: value
  });
}

/**
 * Creates a read-only property for the given member name & the given getter.
 *
 * @param {object}   target - Target object.
 * @param {string}   name   - Member name.
 * @param {mixed}    value  - The attached getter or fixed value.
 */
function readOnlyProperty(target, name, value) {
  var descriptor = {
    enumerable: true,
    configurable: false
  };

  if (typeof value === 'function') {
    descriptor.get = value;
  } else {
    descriptor.value = value;
    descriptor.writable = false;
  }

  Object.defineProperty(target, name, descriptor);
}

/**
 * Function returning uuid v4 compressed into base62 to have 22 characters-long
 * ids easily copy-pastable or usable in a URL.
 *
 * @return {string} - The uuid.
 */
var RANDOM_BYTES = new Uint8Array(16),
    BASE62 = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';

function rng() {
  for (var i = 0, r; i < 16; i++) {
    if ((i & 0x03) === 0) r = Math.random() * 0x100000000;
    RANDOM_BYTES[i] = r >>> ((i & 0x03) << 3) & 0xff;
  }

  return RANDOM_BYTES;
}

function uuidBytes() {
  var random = rng();

  random[6] = random[6] & 0x0f | 0x40;
  random[8] = random[8] & 0x3f | 0x80;

  return random;
}

function toBase62(bytes) {
  var digits = [0];

  for (var i = 0, l = bytes.length; i < l; i++) {
    var carry = bytes[i];

    for (var j = 0, m = digits.length; j < m; j++) {
      carry += digits[j] << 8;
      digits[j] = carry % 62;
      carry = carry / 62 | 0;
    }

    while (carry > 0) {
      digits.push(carry % 62);
      carry = carry / 62 | 0;
    }
  }

  var string = '';

  for (var _i = 0, _l = bytes.length; bytes[_i] === 0 && _i < _l - 1; _i++) {
    string += BASE62[0];
  }for (var _i2 = digits.length - 1; _i2 >= 0; _i2--) {
    string += BASE62[digits[_i2]];
  }while (string.length < 22) {
    string += '0';
  }return string;
}

function uuid() {
  var bytes = uuidBytes();

  return toBase62(bytes);
}