'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = attributes;

var _assert = require('assert');

var _assert2 = _interopRequireDefault(_assert);

var _helpers = require('./helpers');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; } /**
                                                                                                                                                                                                                   * Graphology Attributes Specs
                                                                                                                                                                                                                   * ============================
                                                                                                                                                                                                                   *
                                                                                                                                                                                                                   * Testing the attributes-related methods of the graph.
                                                                                                                                                                                                                   */


var METHODS = ['getNodeAttribute', 'getNodeAttributes', 'hasNodeAttribute', 'getEdgeAttribute', 'getEdgeAttributes', 'hasEdgeAttribute', 'setNodeAttribute', 'setEdgeAttribute', 'updateNodeAttribute', 'updateEdgeAttribute', 'removeNodeAttribute', 'removeEdgeAttribute', 'replaceNodeAttributes', 'replaceEdgeAttributes', 'mergeNodeAttributes', 'mergeEdgeAttributes'];

function attributes(Graph, checkers) {
  var invalid = checkers.invalid,
      notFound = checkers.notFound;


  function commonTests(method) {
    return _defineProperty({}, '#.' + method, {
      'it should throw if the given path is not found.': function itShouldThrowIfTheGivenPathIsNotFound() {
        if (!~method.indexOf('Edge')) return;

        var graph = new Graph();

        _assert2.default.throws(function () {
          graph[method]('source', 'target', 'name', 'value');
        }, notFound());
      },

      'it should throw if the element is not found in the graph.': function itShouldThrowIfTheElementIsNotFoundInTheGraph() {
        var graph = new Graph();

        _assert2.default.throws(function () {
          graph[method]('Test');
        }, notFound());
      }
    });
  }

  var tests = {};

  METHODS.forEach(function (method) {
    return (0, _helpers.deepMerge)(tests, commonTests(method));
  });

  return (0, _helpers.deepMerge)(tests, {
    '#.getAttribute': {
      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        _assert2.default.strictEqual(graph.getAttribute('name'), 'graph');
      },

      'it should return undefined if the attribute does not exist.': function itShouldReturnUndefinedIfTheAttributeDoesNotExist() {
        var graph = new Graph();

        _assert2.default.strictEqual(graph.getAttribute('name'), undefined);
      }
    },

    '#.getNodeAttribute': {

      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.addNode('Martha', { age: 34 });

        _assert2.default.strictEqual(graph.getNodeAttribute('Martha', 'age'), 34);
      },

      'it should return undefined if the attribute does not exist.': function itShouldReturnUndefinedIfTheAttributeDoesNotExist() {
        var graph = new Graph();
        graph.addNode('Martha');

        _assert2.default.strictEqual(graph.getNodeAttribute('Martha', 'age'), undefined);
      }
    },

    '#.getEdgeAttribute': {

      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        var edge = graph.addEdge('John', 'Thomas', { weight: 2 });

        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weight'), 2);
        _assert2.default.strictEqual(graph.getEdgeAttribute('John', 'Thomas', 'weight'), 2);
      },

      'it should return undefined if the attribute does not exist.': function itShouldReturnUndefinedIfTheAttributeDoesNotExist() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        var edge = graph.addEdge('John', 'Thomas');

        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weight'), undefined);
      }
    },

    '#.getAttributes': {

      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        _assert2.default.deepEqual(graph.getAttributes(), { name: 'graph' });
      },

      'it should return an empty object if the node does not have attributes.': function itShouldReturnAnEmptyObjectIfTheNodeDoesNotHaveAttributes() {
        var graph = new Graph();

        _assert2.default.deepEqual(graph.getAttributes(), {});
      }
    },

    '#.getNodeAttributes': {

      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.addNode('Martha', { age: 34 });

        _assert2.default.deepEqual(graph.getNodeAttributes('Martha'), { age: 34 });
      },

      'it should return an empty object if the node does not have attributes.': function itShouldReturnAnEmptyObjectIfTheNodeDoesNotHaveAttributes() {
        var graph = new Graph();
        graph.addNode('Martha');

        _assert2.default.deepEqual(graph.getNodeAttributes('Martha'), {});
      }
    },

    '#.getEdgeAttributes': {

      'it should return the correct value.': function itShouldReturnTheCorrectValue() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        var edge = graph.addEdge('John', 'Thomas', { weight: 2 });

        _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { weight: 2 });
        _assert2.default.deepEqual(graph.getEdgeAttributes('John', 'Thomas'), { weight: 2 });
      },

      'it should return an empty object if the edge does not have attributes.': function itShouldReturnAnEmptyObjectIfTheEdgeDoesNotHaveAttributes() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Thomas']);
        var edge = graph.addEdge('John', 'Thomas');

        _assert2.default.deepEqual(graph.getEdgeAttributes(edge), {});
      }
    },

    '#.hasAttribute': {

      'it should correctly return whether the attribute is set.': function itShouldCorrectlyReturnWhetherTheAttributeIsSet() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        _assert2.default.strictEqual(graph.hasAttribute('name'), true);
        _assert2.default.strictEqual(graph.hasAttribute('info'), false);
      },

      'it does not fail with typical prototypal properties.': function itDoesNotFailWithTypicalPrototypalProperties() {
        var graph = new Graph();

        _assert2.default.strictEqual(graph.hasAttribute('toString'), false);
      }
    },

    '#.hasNodeAttribute': {

      'it should correctly return whether the attribute is set.': function itShouldCorrectlyReturnWhetherTheAttributeIsSet() {
        var graph = new Graph();
        graph.addNode('John', { age: 20 });

        _assert2.default.strictEqual(graph.hasNodeAttribute('John', 'age'), true);
        _assert2.default.strictEqual(graph.hasNodeAttribute('John', 'eyes'), false);
      },

      'it does not fail with typical prototypal properties.': function itDoesNotFailWithTypicalPrototypalProperties() {
        var graph = new Graph();
        graph.addNode('John', { age: 20 });

        _assert2.default.strictEqual(graph.hasNodeAttribute('John', 'toString'), false);
      }
    },

    '#.hasEdgeAttribute': {

      'it should correctly return whether the attribute is set.': function itShouldCorrectlyReturnWhetherTheAttributeIsSet() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        graph.addEdgeWithKey('J->M', 'John', 'Martha', { weight: 10 });

        _assert2.default.strictEqual(graph.hasEdgeAttribute('J->M', 'weight'), true);
        _assert2.default.strictEqual(graph.hasEdgeAttribute('J->M', 'type'), false);
      },

      'it does not fail with typical prototypal properties.': function itDoesNotFailWithTypicalPrototypalProperties() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        graph.addEdgeWithKey('J->M', 'John', 'Martha', { weight: 10 });

        _assert2.default.strictEqual(graph.hasEdgeAttribute('J->M', 'toString'), false);
      }
    },

    '#.setAttribute': {

      'it should correctly set the graph\'s attribute.': function itShouldCorrectlySetTheGraphSAttribute() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        _assert2.default.strictEqual(graph.getAttribute('name'), 'graph');
      }
    },

    '#.setNodeAttribute': {

      'it should correctly set the node\'s attribute.': function itShouldCorrectlySetTheNodeSAttribute() {
        var graph = new Graph();
        graph.addNode('John', { age: 20 });

        graph.setNodeAttribute('John', 'age', 45);
        _assert2.default.strictEqual(graph.getNodeAttribute('John', 'age'), 45);
      }
    },

    '#.setEdgeAttribute': {
      'it should correctly set the edge\'s attribute.': function itShouldCorrectlySetTheEdgeSAttribute() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha', { weigth: 3 });

        graph.setEdgeAttribute(edge, 'weigth', 40);
        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weigth'), 40);

        graph.setEdgeAttribute('John', 'Martha', 'weigth', 60);
        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weigth'), 60);
      }
    },

    '#.updateAttribute': {

      'it should correctly set the graph\'s attribute.': function itShouldCorrectlySetTheGraphSAttribute() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        graph.updateAttribute('name', function (name) {
          return name + '1';
        });
        _assert2.default.strictEqual(graph.getAttribute('name'), 'graph1');
      },

      'the given value should be undefined if not found.': function theGivenValueShouldBeUndefinedIfNotFound() {
        var graph = new Graph();

        var updater = function updater(x) {
          _assert2.default.strictEqual(x, undefined);
          return 'graph';
        };

        graph.updateAttribute('name', updater);
        _assert2.default.strictEqual(graph.getAttribute('name'), 'graph');
      }
    },

    '#.updateNodeAttribute': {

      'it should correctly set the node\'s attribute.': function itShouldCorrectlySetTheNodeSAttribute() {
        var graph = new Graph();
        graph.addNode('John', { age: 20 });

        graph.updateNodeAttribute('John', 'age', function (x) {
          return x + 1;
        });
        _assert2.default.strictEqual(graph.getNodeAttribute('John', 'age'), 21);
      },

      'the given value should be undefined if not found.': function theGivenValueShouldBeUndefinedIfNotFound() {
        var graph = new Graph();
        graph.addNode('John');

        var updater = function updater(x) {
          _assert2.default.strictEqual(x, undefined);
          return 10;
        };

        graph.updateNodeAttribute('John', 'age', updater);
        _assert2.default.strictEqual(graph.getNodeAttribute('John', 'age'), 10);
      }
    },

    '#.updateEdgeAttribute': {
      'it should correctly set the edge\'s attribute.': function itShouldCorrectlySetTheEdgeSAttribute() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha', { weigth: 3 });

        graph.updateEdgeAttribute(edge, 'weigth', function (x) {
          return x + 1;
        });
        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weigth'), 4);

        graph.updateEdgeAttribute('John', 'Martha', 'weigth', function (x) {
          return x + 2;
        });
        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weigth'), 6);
      },

      'the given value should be undefined if not found.': function theGivenValueShouldBeUndefinedIfNotFound() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha');

        var updater = function updater(x) {
          _assert2.default.strictEqual(x, undefined);
          return 10;
        };

        graph.updateEdgeAttribute(edge, 'weight', updater);
        _assert2.default.strictEqual(graph.getEdgeAttribute(edge, 'weight'), 10);
      }
    },

    '#.removeAttribute': {
      'it should correctly remove the attribute.': function itShouldCorrectlyRemoveTheAttribute() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        graph.removeAttribute('name');

        _assert2.default.strictEqual(graph.hasAttribute('name'), false);
        _assert2.default.deepEqual(graph.getAttributes(), {});
      }
    },

    '#.removeNodeAttribute': {
      'it should correctly remove the attribute.': function itShouldCorrectlyRemoveTheAttribute() {
        var graph = new Graph();
        graph.addNode('Martha', { age: 34 });

        graph.removeNodeAttribute('Martha', 'age');

        _assert2.default.strictEqual(graph.hasNodeAttribute('Martha', 'age'), false);
        _assert2.default.deepEqual(graph.getNodeAttributes('Martha'), {});
      }
    },

    '#.removeEdgeAttribute': {
      'it should correclty remove the attribute.': function itShouldCorrecltyRemoveTheAttribute() {
        var graph = new Graph();
        var edge = graph.mergeEdge('John', 'Martha', { weight: 1, size: 3 });

        graph.removeEdgeAttribute('John', 'Martha', 'weight');
        graph.removeEdgeAttribute(edge, 'size');

        _assert2.default.strictEqual(graph.hasEdgeAttribute(edge, 'weight'), false);
        _assert2.default.strictEqual(graph.hasEdgeAttribute(edge, 'size'), false);

        _assert2.default.deepEqual(graph.getEdgeAttributes(edge), {});
      }
    },

    '#.replaceAttribute': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();

        _assert2.default.throws(function () {
          graph.replaceAttributes(true);
        }, invalid());
      },

      'it should correctly replace attributes.': function itShouldCorrectlyReplaceAttributes() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        graph.replaceAttributes({ name: 'other graph' });

        _assert2.default.deepEqual(graph.getAttributes(), { name: 'other graph' });
      }
    },

    '#.replaceNodeAttributes': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();
        graph.addNode('John');

        _assert2.default.throws(function () {
          graph.replaceNodeAttributes('John', true);
        }, invalid());
      },

      'it should correctly replace attributes.': function itShouldCorrectlyReplaceAttributes() {
        var graph = new Graph();
        graph.addNode('John', { age: 45 });

        graph.replaceNodeAttributes('John', { age: 23, eyes: 'blue' });

        _assert2.default.deepEqual(graph.getNodeAttributes('John'), { age: 23, eyes: 'blue' });
      }
    },

    '#.replaceEdgeAttributes': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha');

        _assert2.default.throws(function () {
          graph.replaceEdgeAttributes(edge, true);
        }, invalid());
      },

      'it should correctly replace attributes.': function itShouldCorrectlyReplaceAttributes() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha', { weigth: 1 });

        graph.replaceEdgeAttributes(edge, { weigth: 4, type: 'KNOWS' });

        _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { weigth: 4, type: 'KNOWS' });
      }
    },

    '#.mergeAttributes': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();

        _assert2.default.throws(function () {
          graph.mergeAttributes(true);
        }, invalid());
      },

      'it should correctly merge attributes.': function itShouldCorrectlyMergeAttributes() {
        var graph = new Graph();
        graph.setAttribute('name', 'graph');

        graph.mergeAttributes({ color: 'blue' });

        _assert2.default.deepEqual(graph.getAttributes(), { name: 'graph', color: 'blue' });
      }
    },

    '#.mergeNodeAttributes': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();
        graph.addNode('John');

        _assert2.default.throws(function () {
          graph.mergeNodeAttributes('John', true);
        }, invalid());
      },

      'it should correctly merge attributes.': function itShouldCorrectlyMergeAttributes() {
        var graph = new Graph();
        graph.addNode('John', { age: 45 });

        graph.mergeNodeAttributes('John', { eyes: 'blue' });

        _assert2.default.deepEqual(graph.getNodeAttributes('John'), { age: 45, eyes: 'blue' });
      }
    },

    '#.mergeEdgeAttributes': {
      'it should throw if given attributes are not a plain object.': function itShouldThrowIfGivenAttributesAreNotAPlainObject() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha');

        _assert2.default.throws(function () {
          graph.mergeEdgeAttributes(edge, true);
        }, invalid());
      },

      'it should correctly merge attributes.': function itShouldCorrectlyMergeAttributes() {
        var graph = new Graph();
        graph.addNodesFrom(['John', 'Martha']);
        var edge = graph.addEdge('John', 'Martha', { weigth: 1 });

        graph.mergeEdgeAttributes(edge, { type: 'KNOWS' });

        _assert2.default.deepEqual(graph.getEdgeAttributes(edge), { weigth: 1, type: 'KNOWS' });
      }
    }
  });
}